/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

static GC gc;  /* Graphics Context (necessary for drawing by Xlib) */

/* (3) Event handler called when button is pressed or pointer is moved */
static void DrawEH(w, client_data, event, dispatch)
    Widget     w;        /* Widget on which this event handler was registered */
    XtPointer  client_data;  /* Client data passed at the registration */
    XEvent    *event;     /* Event which triggers this handler */
    Boolean   *dispatch;  /* Further event handler should be called or not */
{
    static int x0, y0;  /* Start point of the line */
    int x, y;           /* End point of the line */

    /*  event->type is event type */
    /*                               */
    if (event->type == ButtonPress &&        /* (4) Button is pressed  */
        event->xbutton.button == Button1) {  /* (4) pressed was Button1 */
        x0 = event->xbutton.x;  /* (5) Initialize X coord. of start point */
        y0 = event->xbutton.y;  /* (5) Initialize Y coord. of start point */
    }
    else if (event->type == MotionNotify &&   /* (6) Pointer is moved  */
        event->xmotion.state & Button1Mask) { /* (6) Button1 is pressed */
        x = event->xmotion.x;  /* (7) X coord. of End point of this line */
        y = event->xmotion.y;  /* (7) Y coord. of End point of this line */

        /* (8) Draw a line from previous pointer location to current one */
        XDrawLine(XtDisplay(w), XtWindow(w), gc, x0, y0, x, y);
        x0 = x; y0 = y;  /* (9) Set the start point of next line */
    }
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, bboard, canvas;
    Arg     al[20];
    int     ac;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Eventhandler", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
    bboard = XmCreateBulletinBoard(toplevel, "bboard", al, ac);
    XtManageChild(bboard);

    ac = 0;
    XtSetArg(al[ac], XmNwidth, 300); ac++;
    XtSetArg(al[ac], XmNheight, 300); ac++;
    canvas = XmCreateDrawingArea(bboard, "canvas", al, ac);
    XtManageChild(canvas);

    /* (1) Register a event handler called when button press and pointer motion */
    XtAddEventHandler(canvas, ButtonPressMask | Button1MotionMask, False,
                      DrawEH, NULL);

    XtRealizeWidget(toplevel);

    /* (2) Create Graphicss context and set the color to black */
    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    XtAppMainLoop(app_context);
}
