/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <X11/keysym.h> /* Header file which defines key symbols */
#include <Xm/XmAll.h>

static GC gc;

/* Callback function called when user input. */
static void InputEventCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    /* (2) copy call_data to cb */
    XmDrawingAreaCallbackStruct *cb = (XmDrawingAreaCallbackStruct *)call_data;
    int x, y;       /* The location the button is pressed */
    KeySym keysym;  /* Key symbol of the inputted key */

    /* cb->event is event struct                    */
    /*         cb->event->type is event type      */
    /*                                          */
    if (cb->event->type == ButtonPress &&        /* (3) button is pressed */
        cb->event->xbutton.button == Button1) {  /* (4) the button is 1 */
        x = cb->event->xbutton.x;  /* (5) the X coord. of pressed location */
        y = cb->event->xbutton.y;  /* (5) the Y coord. of pressed location */
        /* (6) Draw a square where the button is pressed */
        XDrawRectangle(XtDisplay(w), cb->event->xany.window, gc, x, y, 8, 8);
    }
    else if (cb->event->type == KeyPress) {      /* (7) key is pressed    */
        /* (8) Convert keycode to keysym */
        keysym = XKeycodeToKeysym(XtDisplay(w), cb->event->xkey.keycode, 0);

        switch (keysym) {  /* (9) Check the key symbol */
        case XK_a: printf("`a' is pressed\n"); break;
        case XK_1: printf("`1' is pressed\n"); break;
        }
    }
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, bboard, canvas;
    Arg     al[20];
    int     ac;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Inputevent", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
    bboard = XmCreateBulletinBoard(toplevel, "bboard", al, ac);
    XtManageChild(bboard);

    ac = 0;
    XtSetArg(al[ac], XmNwidth, 300); ac++;
    XtSetArg(al[ac], XmNheight, 300); ac++;
    canvas = XmCreateDrawingArea(bboard, "canvas", al, ac);
    XtManageChild(canvas);

    /* (1) Regsiter callback function called when user input */
    XtAddCallback(canvas, XmNinputCallback, InputEventCB, NULL);

    XtRealizeWidget(toplevel);

    /* (1) Create Graphics context and set the color to black */
    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    XtAppMainLoop(app_context);
}
